/* 
 * ==========================================================
 * Artekit demo sketch for the AK-AT30TS75 Temperature sensor
 * breakout board.
 *
 * Copyright (c) 2013 Artekit Italy
 * http://www.artekit.eu
 *
 * Connect the pins in the following way:
 *
 * Arduino UNO          AK-AT30TS75
 * ==================================
 *   AN5                   SCL
 *   AN4                   SDA
 *   Digital 7             ALERT
 *   5V                    VCC
 *   GND                   GND
 *
 * This sketch assumes the AK-AT30TS75 A0,A1 and A2 pins are
 * connected to GND.
 *
 * This demonstration outputs the current 12-bit temperature
 * value and the ALERT pin state. 
 *
 * The ALERT pin will go HIGH when a temperature of 30 Celsius
 * degrees is reached. In this state, the ALERT pin will go back
 * DOWN when the temperature is lower than 29 Celsius degrees.
 *
 * ==========================================================
 */
 
#include <Wire.h>

#define SENSOR_ADDR 0x48

/* This function reads two bytes for the given register */
static unsigned short i2cRead16(byte reg)
{
  unsigned short val;
  Wire.beginTransmission(SENSOR_ADDR);
  Wire.write(reg);
  Wire.endTransmission(true);
  Wire.requestFrom(SENSOR_ADDR, 2);
  
  val = Wire.read() << 8;
  val |= Wire.read();
  return val;
}

/* This function writes two bytes to the given register */
static void i2cWrite16(byte reg, unsigned short value)
{
  Wire.beginTransmission(SENSOR_ADDR);
  Wire.write(reg);
  Wire.write(value >> 8);
  Wire.write(value & 0xFF);
  Wire.endTransmission(true);
}
 
/* This function writes a byte to the given register */
static void i2cWrite8(byte reg, byte value)
{
  Wire.beginTransmission(SENSOR_ADDR);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission(true);
}

void sensorConfig()
{
  /* 12-bit resolution, ALARM pin is active-high */
  i2cWrite8(0x01, 0x64);
  
  /* 12-bit temperature is read/written using the formula:
   * 
   * (temp / 0.0625) << 4
   *
   * temp = temperature
   * 0.0625 = resolution for 12-bit values
   */
   
  /* 29 degrees for T-low threshold */
  i2cWrite16(0x02, 0x1D00);
  
  /* 30 degrees for T-high threshold */
  i2cWrite16(0x03, 0x1E00);
}

void sensorRead()
{
  unsigned short val;
  float temperature;

  Serial.print("Temp=");
  temperature = (i2cRead16(0x00) >> 4);
  temperature *= 0.0625f;
  Serial.print(temperature);
  
  Serial.print(" Alarm=");
  Serial.println(digitalRead(7));
}

void setup()
{
  /* I2C start-up */
  Wire.begin();
  
  /* Serial start-up at 9600bps */
  Serial.begin(9600);
  
  /* Set the Digital 7 pin as input */
  pinMode(7, INPUT);
  
  /* Activate the pull-up resistor on Digital 7 pin */
  digitalWrite(7, HIGH);
  
  /* Configure the temperature sensor */
  sensorConfig();
}
 
void loop()
{
  /* Read the temperature each second */
  sensorRead();
  delay(1000); 
}
